#include <graphics/ProgressBar.h>

ProgressBar::ProgressBar( float x, float y, unsigned int width, unsigned int height, FrameImage *imgFrame,
		Sprite spBar, float max):
Frame(x, y, width, height, imgFrame),
_Elapsed		(0.0f),
_Max			(max),
_ReductionEffect(false),
_ReductionPos	(0),
_FadingSpeed	(150.f)
{
	_BarPos = Vector2f(x + static_cast<float>(GetClientX()),
		y + static_cast<float>(GetClientY()));

	_spBar = spBar;
	_spBar.Resize(0.f,
		static_cast<float>(GetClientHeight()));
	_spBar.SetPosition(_BarPos);

	// Reduction effect
	_ReductionEffectEnable = false;
	_imgReduction.Create(1, 1, Color::White);

	_spReduction = Sprite(_imgReduction);
	_spReduction.Resize(0.f,
		static_cast<float>(GetClientHeight()));
	_spReduction.SetPosition(_BarPos);
}

ProgressBar::~ProgressBar()
{
}

void ProgressBar::Restart()
{
	_Elapsed = 0.0f;
	_spBar.Resize(0.f, static_cast<float>(GetClientHeight()));
}

bool ProgressBar::FrameMove(float elapsedTime)
{
	// Fading effect
	if (_ReductionEffect)
	{
		Color col = _spReduction.GetColor();
		float alpha = col.a;
		alpha -= elapsedTime * _FadingSpeed;
		if (alpha <= 0)
		{
			_ReductionEffect = false;
			alpha = 0;
		}
		else
		{
			col.a = static_cast<sf::Uint8>(alpha);
			_spReduction.SetColor(col);
		}
	}

	return false;
}

void ProgressBar::Render(sf::RenderWindow &screen)
{
	Frame::Render(screen);
	if (_ReductionEffect)
	{
		screen.Draw(_spReduction);
	}

	screen.Draw(_spBar);
}

void ProgressBar::Render(sf::RenderWindow &screen, sf::Vector2f offset)
{
	Frame::Render(screen, offset.x, offset.y);

	if (_ReductionEffect)
	{
		_spReduction.SetPosition(_ReductionPos + offset.x, _BarPos.y + offset.y);
		screen.Draw(_spReduction);
	}

	_spBar.SetPosition(_BarPos.x + offset.x, _BarPos.y + offset.y);
	screen.Draw(_spBar);
}

bool ProgressBar::Add(float value)
{
	_Elapsed += value;
	if (_Elapsed > _Max)
	{
		_Elapsed = _Max;
	}
	ScaleProgressBar();

	return _Elapsed == _Max;
}
void ProgressBar::Substract(float value)
{
	float prevTime = _Elapsed;
	_Elapsed -= value;
	if (_Elapsed < 0.0f)
		_Elapsed = 0.0f;
	ScaleProgressBar();

	if (_ReductionEffectEnable)
	{
		_ReductionEffect = true;
		_spReduction.Resize((prevTime - _Elapsed) * static_cast<float>(GetClientWidth()) / _Max,
			_spReduction.GetSize().y);
		_ReductionPos = _BarPos.x + (_Elapsed * static_cast<float>(GetClientWidth())) / _Max;
		_spReduction.SetPosition(_ReductionPos, _spReduction.GetPosition().y);

		Color col = _spReduction.GetColor();
		col.a = 255;
		_spReduction.SetColor(col);
	}
}

float ProgressBar::GetRemaining() const
{
	return _Max - _Elapsed;
}

void ProgressBar::SetElapsed(float value)
{
	if( value < _Elapsed)
		Substract(_Elapsed - value);
	else
		Add(value - _Elapsed);
}

float ProgressBar::GetElapsed() const
{
	return _Elapsed;
}

void ProgressBar::SetMax( const float &value)
{
	_Max = value;
}

float ProgressBar::GetMax(void) const
{
	return _Max;
}

// Fading when the progress bar is substracted
// Enable the effect and set the color of the fading
void ProgressBar::AddReductionEffect(Color col)
{
	_ReductionEffectEnable = true;
	_imgReduction.Create(1, 1, col);
}

// Enable or disable the reduction effect
void ProgressBar::EnableReductionEffect(bool enable)
{
	_ReductionEffectEnable = enable;
}

void ProgressBar::SetFadingSpeed(const float fadingSpeed)
{
	_FadingSpeed = fadingSpeed;
}

float ProgressBar::GetFadingSpeed() const
{
	return _FadingSpeed;
}

// Private functions ======================================

// Scale the sprite representing the progression
void ProgressBar::ScaleProgressBar()
{
	// Dimensionnement de la barre de temps coul
	float newWidth = (_Elapsed * static_cast<float>(GetClientWidth())) / _Max;
	_spBar.Resize(newWidth, _spBar.GetSize().y);
}
